class OmnicommError(Exception):
    ...


class EmptyDataError(OmnicommError):
    ...


class CommandAlreadyExistsError(OmnicommError):
    ...


class UnpackingCRCError(OmnicommError):
    ...


class CRCDoesNotMatchError(OmnicommError):
    ...


class FrameMarkerDoesNotExistError(OmnicommError):
    ...


class ProtoDoesNotExistError(OmnicommError):
    ...



import time
from unittest import TestCase

from omnicomm import commands, exceptions, protocol


class ProtocolTest(TestCase):
    def setUp(self) -> None:
        protocol.Protocol.load_command_proto(
            {
                0x9F: 'omnicomm.proto.profi_optim_lite_pb2.RecReg',
            }
        )

    def test_unpack_empty_data(self):
        with self.assertRaises(exceptions.EmptyDataError):
            protocol.Protocol.unpack(b'')
        with self.assertRaises(exceptions.EmptyDataError):
            protocol.ServerProtocol.unpack(b'')
        with self.assertRaises(exceptions.EmptyDataError):
            protocol.RegistrarProtocol.unpack(b'')

    def test_80(self):
        value = {'reg_id': 202000013, 'firmware': 114}
        data = protocol.RegistrarProtocol.pack(commands.Cmd80(value))
        cmd, remain = protocol.ServerProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd80.id)
        self.assertTrue(cmd.value == value)

    def test_85(self):
        value = {'rec_id': 62311}
        data = protocol.ServerProtocol.pack(commands.Cmd85(value))
        cmd, remain = protocol.RegistrarProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd85.id)
        self.assertTrue(cmd.value == value)

    def test_86(self):
        value = {
            'rec_id': 0,
            'omnicomm_time': 0,
            'unix_time': commands.Cmd85.BASE_TIME,
            'priority': 0,
            'msgs': [
                {
                    'mID': [1],
                    'general': {
                        'Time': 135433134,
                        'FLG': 7,
                        'Mileage': 0,
                        'VImp': 0,
                        'TImp': 1000,
                        'Uboard': 124,
                        'SumAcc': 102,
                    },
                    'nav': {
                        'LAT': 557887999,
                        'LON': 375883499,
                        'GPSVel': 3,
                        'GPSDir': 264,
                        'GPSNSat': 8,
                        'GPSAlt': 2096,
                    },
                },
            ],
        }
        data = protocol.RegistrarProtocol.pack(commands.Cmd86(value))
        cmd, remain = protocol.ServerProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd86.id)
        self.assertTrue(cmd.value == value)

    def test_87(self):
        value = {'rec_id': 123}
        data = protocol.ServerProtocol.pack(commands.Cmd87(value))
        cmd, remain = protocol.RegistrarProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd87.id)
        self.assertTrue(cmd.value == value)

    def test_88(self):
        value = {'rec_id': 123}
        data = protocol.RegistrarProtocol.pack(commands.Cmd88(value))
        cmd, remain = protocol.ServerProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd88.id)
        self.assertTrue(cmd.value == value)

    def test_93_from_server(self):
        value = {}
        data = protocol.ServerProtocol.pack(commands.Cmd93(value))
        cmd, remain = protocol.RegistrarProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd93.id)
        self.assertTrue(cmd.value == value)

    def test_93_from_registrar(self):
        value = {}
        data = protocol.RegistrarProtocol.pack(commands.Cmd93(value))
        cmd, remain = protocol.ServerProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd93.id)
        self.assertTrue(cmd.value == value)

    def test_94_from_server(self):
        value = {'unix_time': int(time.time())}
        data = protocol.ServerProtocol.pack(commands.Cmd94(value))
        cmd, remain = protocol.RegistrarProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd94.id)
        self.assertTrue(cmd.value['unix_time'] == value['unix_time'])

    def test_94_from_registrar(self):
        value = {'unix_time': int(time.time())}
        data = protocol.RegistrarProtocol.pack(commands.Cmd94(value))
        cmd, remain = protocol.ServerProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd94.id)
        self.assertTrue(cmd.value['unix_time'] == value['unix_time'])

    def test_95(self):
        value = {
            'rec_id': 0,
            'omnicomm_time': 0,
            'unix_time': commands.Cmd95.BASE_TIME,
            'priority': 0,
            'msgs': [
                {
                    'mID': [1],
                    'general': {
                        'Time': 135433134,
                        'FLG': 7,
                        'Mileage': 0,
                        'VImp': 0,
                        'TImp': 1000,
                        'Uboard': 124,
                        'SumAcc': 102,
                    },
                    'nav': {
                        'LAT': 557887999,
                        'LON': 375883499,
                        'GPSVel': 3,
                        'GPSDir': 264,
                        'GPSNSat': 8,
                        'GPSAlt': 2096,
                    },
                },
            ],
        }
        data = protocol.RegistrarProtocol.pack(commands.Cmd95(value))
        cmd, remain = protocol.ServerProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd95.id)
        self.assertTrue(cmd.value == value)

    def test_9f(self):
        value = {
            'rec_id': 0,
            'omnicomm_time': 0,
            'unix_time': commands.Cmd9F.BASE_TIME,
            'priority': 0,
            'msgs': [
                {
                    'mID': [1],
                    'general': {
                        'Time': 135433134,
                        'FLG': 7,
                        'Mileage': 0,
                        'VImp': 0,
                        'TImp': 1000,
                        'Uboard': 124,
                        'SumAcc': 102,
                    },
                    'nav': {
                        'LAT': 557887999,
                        'LON': 375883499,
                        'GPSVel': 3,
                        'GPSDir': 264,
                        'GPSNSat': 8,
                        'GPSAlt': 2096,
                    },
                },
            ],
        }
        data = protocol.RegistrarProtocol.pack(commands.Cmd9F(value))
        cmd, remain = protocol.ServerProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.Cmd9F.id)
        self.assertTrue(cmd.value == value)

    def test_a0(self):
        value = {'rec_id': 123}
        data = protocol.ServerProtocol.pack(commands.CmdA0(value))
        cmd, remain = protocol.RegistrarProtocol.unpack(data)
        self.assertTrue(cmd.id == commands.CmdA0.id)
        self.assertTrue(cmd.value == value)


class ProtocolUnpackMany(TestCase):
    def test_cmd_one_no_remain(self):
        value = {'reg_id': 1234567890, 'firmware': 1}
        data = protocol.RegistrarProtocol.pack(commands.Cmd80(value))
        original_cmds = [data]
        original_remain = b''
        cmds, remain = protocol.ServerProtocol.unpack_many(b''.join(original_cmds))
        self.assertTrue(len(cmds) == len(original_cmds))
        self.assertTrue(remain == original_remain)

    def test_cmd_one_with_remain(self):
        value = {'reg_id': 1234567890, 'firmware': 1}
        data = protocol.RegistrarProtocol.pack(commands.Cmd80(value))
        original_cmds = [data]
        original_remain = data[:-1]
        cmds, remain = protocol.ServerProtocol.unpack_many(b''.join(original_cmds) + original_remain)
        self.assertTrue(len(cmds) == len(original_cmds))
        self.assertTrue(remain == original_remain)

    def test_cmd_many_no_remain(self):
        value = {'reg_id': 1234567890, 'firmware': 1}
        data = protocol.RegistrarProtocol.pack(commands.Cmd80(value))
        original_cmds = [data, data]
        original_remain = b''
        cmds, remain = protocol.ServerProtocol.unpack_many(b''.join(original_cmds))
        self.assertTrue(len(cmds) == len(original_cmds))
        self.assertTrue(remain == original_remain)

    def test_cmd_many_with_remain(self):
        value = {'reg_id': 1234567890, 'firmware': 1}
        data = protocol.RegistrarProtocol.pack(commands.Cmd80(value))
        original_cmds = [data, data]
        original_remain = data[:-1]
        cmds, remain = protocol.ServerProtocol.unpack_many(b''.join(original_cmds) + original_remain)
        self.assertTrue(len(cmds) == len(original_cmds))
        self.assertTrue(remain == original_remain)

    def test_empty_data_error(self):
        with self.assertRaises(exceptions.EmptyDataError):
            protocol.ServerProtocol.unpack_many(b'')

    def test_frame_marker_does_not_exist_error(self):
        value = {'reg_id': 1234567890, 'firmware': 1}
        data = protocol.RegistrarProtocol.pack(commands.Cmd80(value))
        with self.assertRaises(exceptions.FrameMarkerDoesNotExistError):
            protocol.ServerProtocol.unpack_many(data[1:])

    def test_unpacking_crc_error(self):
        value = {'reg_id': 1234567890, 'firmware': 1}
        data = protocol.RegistrarProtocol.pack(commands.Cmd80(value))
        with self.assertRaises(exceptions.UnpackingCRCError):
            protocol.ServerProtocol.unpack_many(data[:-1])

    def test_crc_does_not_match_error(self):
        value = {'reg_id': 1234567890, 'firmware': 1}
        data = protocol.RegistrarProtocol.pack(commands.Cmd80(value))
        data = data[:-1] + bytes.fromhex(hex(data[-1] - 1)[2:])
        with self.assertRaises(exceptions.CRCDoesNotMatchError):
            protocol.ServerProtocol.unpack_many(data)



from unittest import TestCase

from omnicomm import commands, protocol


class Cmd85Test(TestCase):
    def test_simple(self):
        msg = bytes.fromhex('67F30000')
        cmd85 = commands.Cmd85.unpack(msg)
        self.assertTrue(commands.Cmd85(cmd85.value).pack() == msg)
        value = {'rec_id': 62311}
        data = commands.Cmd85(value).pack()
        self.assertTrue(commands.Cmd85.unpack(data).value == value)


class Cmd86Test(TestCase):
    def setUp(self) -> None:
        protocol.Protocol.load_command_proto()

    def test_simple(self):
        msg = bytes.fromhex(
            '67f30000ae8b12080044000a01011308ae97ca4020072800300038e807407c50'
            'cc01142b08fec785940410d69bbce6021803208802280830e0202c3308001000'
            '18e20220003443083810be10180444'
        )
        cmd86 = commands.Cmd86.unpack(msg)
        self.assertTrue(commands.Cmd86(cmd86.value).pack() == msg)

    def test_empty(self):
        cmd86 = commands.Cmd86.unpack(b'')
        self.assertTrue(cmd86.value == {})
        self.assertTrue(commands.Cmd86(cmd86.value).pack() == b'')

        data = commands.Cmd86({'rec_id': 1, 'omnicomm_time': 2, 'priority': 0}).pack()
        self.assertTrue(commands.Cmd86.unpack(data).value == {})

        data = commands.Cmd86({'rec_id': 1, 'unix_time': commands.Cmd86.BASE_TIME, 'priority': 0}).pack()
        self.assertTrue(commands.Cmd86.unpack(data).value == {})


class Cmd87Test(TestCase):
    def test_simple(self):
        value = {'rec_id': 123}
        data = commands.Cmd87(value).pack()
        self.assertTrue(commands.Cmd87.unpack(data).value == value)


class Cmd88Test(TestCase):
    def test_simple(self):
        value = {'rec_id': 456}
        data = commands.Cmd88(value).pack()
        self.assertTrue(commands.Cmd88.unpack(data).value == value)


class Cmd93Test(TestCase):
    def test_simple(self):
        msg = bytes.fromhex('')
        cmd93 = commands.Cmd93.unpack(msg)
        self.assertTrue(commands.Cmd93(cmd93.value).pack() == msg)
        value = {}
        data = commands.Cmd93(value).pack()
        self.assertTrue(commands.Cmd93.unpack(data).value == value)


class Cmd94Test(TestCase):
    def test_simple(self):
        msg = bytes.fromhex('c4a6b51b')
        cmd94 = commands.Cmd94.unpack(msg)
        self.assertTrue(commands.Cmd94(cmd94.value).pack() == msg)
        value1 = {'omnicomm_time': 464889540}
        data = commands.Cmd94(value1).pack()
        self.assertTrue(commands.Cmd94.unpack(data).value['omnicomm_time'] == value1['omnicomm_time'])
        value2 = {'unix_time': 1695657540}
        data = commands.Cmd94(value2).pack()
        self.assertTrue(commands.Cmd94.unpack(data).value['omnicomm_time'] == value1['omnicomm_time'])


class Cmd95Test(TestCase):
    def setUp(self) -> None:
        protocol.Protocol.load_command_proto()

    def test_simple(self):
        msg = bytes.fromhex(
            '67f30000ae8b12080044000a01011308ae97ca4020072800300038e807407c50'
            'cc01142b08fec785940410d69bbce6021803208802280830e0202c3308001000'
            '18e20220003443083810be10180444'
        )
        cmd95 = commands.Cmd95.unpack(msg)
        self.assertTrue(commands.Cmd86(cmd95.value).pack() == msg)


class Cmd9FTest(TestCase):
    def setUp(self) -> None:
        protocol.Protocol.load_command_proto(
            {
                0x9F: 'omnicomm.proto.profi_optim_lite_pb2.RecReg',
            }
        )

    def test_simple(self):
        msg = bytes.fromhex(
            '67f30000ae8b12080044000a01011308ae97ca4020072800300038e807407c50'
            'cc01142b08fec785940410d69bbce6021803208802280830e0202c3308001000'
            '18e20220003443083810be10180444'
        )
        cmd9f = commands.Cmd9F.unpack(msg)
        self.assertTrue(commands.Cmd86(cmd9f.value).pack() == msg)


class CmdA0Test(TestCase):
    def test_simple(self):
        value = {'rec_id': 123}
        data = commands.CmdA0(value).pack()
        self.assertTrue(commands.CmdA0.unpack(data).value == value)



import random
import socket
import socketserver
import time
import unittest
from collections.abc import Generator
from contextlib import contextmanager
from queue import Queue
from threading import Thread

from omnicomm import commands
from omnicomm.protocol import RegistrarProtocol, ServerProtocol


class SimpleTCPServer(socketserver.ThreadingTCPServer):
    allow_reuse_address = True
    request_queue_size = 1000
    daemon_threads = True


class SimpleTCPRequestHandler(socketserver.BaseRequestHandler):
    def handle(self):
        data = self.request.recv(1024)
        cmd80, remain = ServerProtocol.unpack(data)

        data = {'rec_id': random.randint(1, 100)}  # noqa: S311
        data = ServerProtocol.pack(commands.Cmd85(data))
        self.request.sendall(data)

        data = self.request.recv(1024)
        cmd86, remain = ServerProtocol.unpack(data)

        data = self.request.recv(1024)
        cmd86, remain = ServerProtocol.unpack(data)

        data = {'rec_id': random.randint(1, 100)}  # noqa: S311
        data = ServerProtocol.pack(commands.Cmd87(data))
        self.request.sendall(data)

        data = self.request.recv(1024)
        cmd88, remain = ServerProtocol.unpack(data)


Server = type[socketserver.ThreadingTCPServer]
ServerHandler = type[socketserver.BaseRequestHandler]


class Registrar(Thread):
    def __init__(self, host: str, port: int, q_input: Queue, q_output: Queue) -> None:
        self.host = host
        self.port = port
        self.q_input = q_input
        self.q_output = q_output

        super().__init__(daemon=True)

    def run(self):
        tasks = self.q_input.get()

        cmd = commands.Cmd80(dict(reg_id=0, firmware=tasks))
        data = RegistrarProtocol.pack(cmd)

        tcp_client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        tcp_client.connect((self.host, self.port))
        tcp_client.sendall(data)

        received = tcp_client.recv(1024)
        cmd85, remain = RegistrarProtocol.unpack(received)

        data = {
            'rec_id': 0,
            'omnicomm_time': 0,
            'priority': 0,
            'msgs': [
                {
                    'mID': [0x0B],
                    'general': {
                        'Time': 135433134,
                        'FLG': 7,
                        'Mileage': 0,
                        'VImp': 0,
                        'TImp': 1000,
                        'Uboard': 124,
                        'SumAcc': 102,
                    },
                    'nav': {
                        'LAT': 557887999,
                        'LON': 375883499,
                        'GPSVel': 3,
                        'GPSDir': 264,
                        'GPSNSat': 8,
                        'GPSAlt': 2096,
                    },
                },
            ],
        }
        cmd86 = commands.Cmd86(data)
        data = RegistrarProtocol.pack(cmd86)
        tcp_client.send(data)

        cmd86 = commands.Cmd86({})
        data = RegistrarProtocol.pack(cmd86)
        tcp_client.sendall(data)

        received = tcp_client.recv(1024)
        cmd87, remain = RegistrarProtocol.unpack(received)

        time.sleep(1)

        data = RegistrarProtocol.pack(commands.Cmd88(cmd87.value))
        tcp_client.sendall(data)

        time.sleep(0.1)

        self.q_output.put(1)

        self.q_input.task_done()


class TestServer(unittest.TestCase):
    @staticmethod
    @contextmanager
    def server(server_cls: Server, handler_cls: ServerHandler, host: str, port: int) -> Generator:
        server = server_cls((host, port), handler_cls)
        Thread(target=server.serve_forever, daemon=True).start()
        yield server
        server.server_close()

    def test_load(self):
        ServerProtocol.load_command_proto()

        q_input = Queue()
        q_output = Queue()

        host, port = 'localhost', 38700
        clients = 1
        messages = 1

        for _ in range(clients):
            q_input.put(messages)

        with self.server(SimpleTCPServer, SimpleTCPRequestHandler, host, port):
            for _ in range(clients):
                Registrar(host, port, q_input, q_output).start()

            q_input.join()

        self.assertTrue(len(list(filter(bool, q_output.queue))) == clients * messages)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: profi_optim_lite.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='profi_optim_lite.proto',
  package='profi_optim_lite',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x16profi_optim_lite.proto\x12\x10profi_optim_lite\"\xb6\x0c\n\x06RecReg\x12\x0f\n\x03mID\x18\x01 \x03(\rB\x02\x10\x01\x12\x31\n\x07general\x18\x02 \x01(\n2 .profi_optim_lite.RecReg.General\x12-\n\x05photo\x18\x04 \x01(\n2\x1e.profi_optim_lite.RecReg.Photo\x12)\n\x03nav\x18\x05 \x01(\n2\x1c.profi_optim_lite.RecReg.NAV\x12-\n\x05unidt\x18\x06 \x01(\n2\x1e.profi_optim_lite.RecReg.UniDt\x12-\n\x05\x63\x61ndt\x18\x07 \x01(\n2\x1e.profi_optim_lite.RecReg.CanDt\x12-\n\x05llsdt\x18\x08 \x01(\n2\x1e.profi_optim_lite.RecReg.LLSDt\x12)\n\x03log\x18\x10 \x01(\n2\x1c.profi_optim_lite.RecReg.LOG\x1a\xdc\x01\n\x07General\x12\x0c\n\x04Time\x18\x01 \x01(\r\x12\r\n\x05IDFAS\x18\x02 \x01(\r\x12\r\n\x05IDDrv\x18\x03 \x01(\x0c\x12\x0b\n\x03\x46LG\x18\x04 \x01(\r\x12\x0f\n\x07Mileage\x18\x05 \x01(\r\x12\x0c\n\x04VImp\x18\x06 \x01(\r\x12\x0c\n\x04TImp\x18\x07 \x01(\r\x12\x0e\n\x06Uboard\x18\x08 \x01(\r\x12\x0f\n\x07\x42\x61tLife\x18\t \x01(\r\x12\x0e\n\x06SumAcc\x18\n \x01(\x11\x12\r\n\x05Phone\x18\x0b \x01(\x0c\x12\r\n\x05\x41mtrX\x18\x0c \x01(\x11\x12\r\n\x05\x41mtrY\x18\r \x01(\x11\x12\r\n\x05\x41mtrZ\x18\x0e \x01(\x11\x1a\x62\n\x05Photo\x12\x0e\n\x06POSBLK\x18\x01 \x01(\r\x12\x0f\n\x07SZPHOTO\x18\x02 \x01(\r\x12\x0c\n\x04SIZE\x18\x03 \x01(\r\x12\x0b\n\x03\x42UF\x18\x04 \x01(\x0c\x12\x0c\n\x04IDPH\x18\x05 \x01(\r\x12\x0f\n\x07IMGSTAT\x18\x06 \x01(\r\x1a`\n\x03NAV\x12\x0b\n\x03LAT\x18\x01 \x02(\x11\x12\x0b\n\x03LON\x18\x02 \x02(\x11\x12\x0e\n\x06GPSVel\x18\x03 \x02(\r\x12\x0e\n\x06GPSDir\x18\x04 \x02(\r\x12\x0f\n\x07GPSNSat\x18\x05 \x02(\r\x12\x0e\n\x06GPSAlt\x18\x06 \x02(\x11\x1aK\n\x05UniDt\x12\x0f\n\x07UniVal0\x18\x01 \x01(\x11\x12\x0f\n\x07UniVal1\x18\x02 \x01(\x11\x12\x0f\n\x07UniVal2\x18\x03 \x01(\x11\x12\x0f\n\x07UniVal3\x18\x04 \x01(\x11\x1a\xb5\x03\n\x05\x43\x61nDt\x12\r\n\x05SPN70\x18\x46 \x01(\r\x12\r\n\x05SPN91\x18[ \x01(\r\x12\x0e\n\x06SPN100\x18\x64 \x01(\r\x12\x0e\n\x06SPN110\x18n \x01(\r\x12\x0f\n\x06SPN174\x18\xae\x01 \x01(\r\x12\x0f\n\x06SPN175\x18\xaf\x01 \x01(\r\x12\x0f\n\x06SPN182\x18\xb6\x01 \x01(\r\x12\x0f\n\x06SPN184\x18\xb8\x01 \x01(\r\x12\x0f\n\x06SPN190\x18\xbe\x01 \x01(\r\x12\x0f\n\x06SPN244\x18\xf4\x01 \x01(\r\x12\x0f\n\x06SPN245\x18\xf5\x01 \x01(\r\x12\x0f\n\x06SPN247\x18\xf7\x01 \x01(\r\x12\x0f\n\x06SPN250\x18\xfa\x01 \x01(\r\x12\x0f\n\x06SPN521\x18\x89\x04 \x01(\r\x12\x0f\n\x06SPN522\x18\x8a\x04 \x01(\r\x12\x0f\n\x06SPN527\x18\x8f\x04 \x01(\r\x12\x13\n\x06SPN582\x18\xc6\x04 \x03(\rB\x02\x10\x01\x12\x0f\n\x06SPN597\x18\xd5\x04 \x01(\r\x12\x0f\n\x06SPN598\x18\xd6\x04 \x01(\r\x12\x0f\n\x06SPN914\x18\x92\x07 \x01(\r\x12\x0f\n\x06SPN916\x18\x94\x07 \x01(\r\x12\x13\n\x06SPN928\x18\xa0\x07 \x03(\rB\x02\x10\x01\x12\x10\n\x07SPN1624\x18\xd8\x0c \x01(\r\x12\x10\n\x07SPN1821\x18\x9d\x0e \x01(\r\x12\x10\n\x07SPN1856\x18\xc0\x0e \x01(\r\x1a\x95\x02\n\x05LLSDt\x12\r\n\x05TLLS1\x18\x01 \x01(\x11\x12\r\n\x05\x43LLS1\x18\x02 \x01(\r\x12\r\n\x05\x46LLS1\x18\x03 \x01(\x11\x12\r\n\x05TLLS2\x18\x04 \x01(\x11\x12\r\n\x05\x43LLS2\x18\x05 \x01(\r\x12\r\n\x05\x46LLS2\x18\x06 \x01(\x11\x12\r\n\x05TLLS3\x18\x07 \x01(\x11\x12\r\n\x05\x43LLS3\x18\x08 \x01(\r\x12\r\n\x05\x46LLS3\x18\t \x01(\x11\x12\r\n\x05TLLS4\x18\n \x01(\x11\x12\r\n\x05\x43LLS4\x18\x0b \x01(\r\x12\r\n\x05\x46LLS4\x18\x0c \x01(\x11\x12\r\n\x05TLLS5\x18\r \x01(\x11\x12\r\n\x05\x43LLS5\x18\x0e \x01(\r\x12\r\n\x05\x46LLS5\x18\x0f \x01(\x11\x12\r\n\x05TLLS6\x18\x10 \x01(\x11\x12\r\n\x05\x43LLS6\x18\x11 \x01(\r\x12\r\n\x05\x46LLS6\x18\x12 \x01(\x11\x1a\x14\n\x03LOG\x12\r\n\x05\x44\x65\x62ug\x18\x01 \x03(\t'
)




_RECREG_GENERAL = _descriptor.Descriptor(
  name='General',
  full_name='profi_optim_lite.RecReg.General',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='Time', full_name='profi_optim_lite.RecReg.General.Time', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='IDFAS', full_name='profi_optim_lite.RecReg.General.IDFAS', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='IDDrv', full_name='profi_optim_lite.RecReg.General.IDDrv', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='FLG', full_name='profi_optim_lite.RecReg.General.FLG', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='Mileage', full_name='profi_optim_lite.RecReg.General.Mileage', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='VImp', full_name='profi_optim_lite.RecReg.General.VImp', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='TImp', full_name='profi_optim_lite.RecReg.General.TImp', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='Uboard', full_name='profi_optim_lite.RecReg.General.Uboard', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='BatLife', full_name='profi_optim_lite.RecReg.General.BatLife', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SumAcc', full_name='profi_optim_lite.RecReg.General.SumAcc', index=9,
      number=10, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='Phone', full_name='profi_optim_lite.RecReg.General.Phone', index=10,
      number=11, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='AmtrX', full_name='profi_optim_lite.RecReg.General.AmtrX', index=11,
      number=12, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='AmtrY', full_name='profi_optim_lite.RecReg.General.AmtrY', index=12,
      number=13, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='AmtrZ', full_name='profi_optim_lite.RecReg.General.AmtrZ', index=13,
      number=14, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=398,
  serialized_end=618,
)

_RECREG_PHOTO = _descriptor.Descriptor(
  name='Photo',
  full_name='profi_optim_lite.RecReg.Photo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='POSBLK', full_name='profi_optim_lite.RecReg.Photo.POSBLK', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SZPHOTO', full_name='profi_optim_lite.RecReg.Photo.SZPHOTO', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SIZE', full_name='profi_optim_lite.RecReg.Photo.SIZE', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='BUF', full_name='profi_optim_lite.RecReg.Photo.BUF', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='IDPH', full_name='profi_optim_lite.RecReg.Photo.IDPH', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='IMGSTAT', full_name='profi_optim_lite.RecReg.Photo.IMGSTAT', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=620,
  serialized_end=718,
)

_RECREG_NAV = _descriptor.Descriptor(
  name='NAV',
  full_name='profi_optim_lite.RecReg.NAV',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='LAT', full_name='profi_optim_lite.RecReg.NAV.LAT', index=0,
      number=1, type=17, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='LON', full_name='profi_optim_lite.RecReg.NAV.LON', index=1,
      number=2, type=17, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='GPSVel', full_name='profi_optim_lite.RecReg.NAV.GPSVel', index=2,
      number=3, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='GPSDir', full_name='profi_optim_lite.RecReg.NAV.GPSDir', index=3,
      number=4, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='GPSNSat', full_name='profi_optim_lite.RecReg.NAV.GPSNSat', index=4,
      number=5, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='GPSAlt', full_name='profi_optim_lite.RecReg.NAV.GPSAlt', index=5,
      number=6, type=17, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=720,
  serialized_end=816,
)

_RECREG_UNIDT = _descriptor.Descriptor(
  name='UniDt',
  full_name='profi_optim_lite.RecReg.UniDt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='UniVal0', full_name='profi_optim_lite.RecReg.UniDt.UniVal0', index=0,
      number=1, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='UniVal1', full_name='profi_optim_lite.RecReg.UniDt.UniVal1', index=1,
      number=2, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='UniVal2', full_name='profi_optim_lite.RecReg.UniDt.UniVal2', index=2,
      number=3, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='UniVal3', full_name='profi_optim_lite.RecReg.UniDt.UniVal3', index=3,
      number=4, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=818,
  serialized_end=893,
)

_RECREG_CANDT = _descriptor.Descriptor(
  name='CanDt',
  full_name='profi_optim_lite.RecReg.CanDt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='SPN70', full_name='profi_optim_lite.RecReg.CanDt.SPN70', index=0,
      number=70, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN91', full_name='profi_optim_lite.RecReg.CanDt.SPN91', index=1,
      number=91, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN100', full_name='profi_optim_lite.RecReg.CanDt.SPN100', index=2,
      number=100, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN110', full_name='profi_optim_lite.RecReg.CanDt.SPN110', index=3,
      number=110, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN174', full_name='profi_optim_lite.RecReg.CanDt.SPN174', index=4,
      number=174, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN175', full_name='profi_optim_lite.RecReg.CanDt.SPN175', index=5,
      number=175, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN182', full_name='profi_optim_lite.RecReg.CanDt.SPN182', index=6,
      number=182, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN184', full_name='profi_optim_lite.RecReg.CanDt.SPN184', index=7,
      number=184, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN190', full_name='profi_optim_lite.RecReg.CanDt.SPN190', index=8,
      number=190, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN244', full_name='profi_optim_lite.RecReg.CanDt.SPN244', index=9,
      number=244, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN245', full_name='profi_optim_lite.RecReg.CanDt.SPN245', index=10,
      number=245, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN247', full_name='profi_optim_lite.RecReg.CanDt.SPN247', index=11,
      number=247, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN250', full_name='profi_optim_lite.RecReg.CanDt.SPN250', index=12,
      number=250, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN521', full_name='profi_optim_lite.RecReg.CanDt.SPN521', index=13,
      number=521, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN522', full_name='profi_optim_lite.RecReg.CanDt.SPN522', index=14,
      number=522, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN527', full_name='profi_optim_lite.RecReg.CanDt.SPN527', index=15,
      number=527, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN582', full_name='profi_optim_lite.RecReg.CanDt.SPN582', index=16,
      number=582, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN597', full_name='profi_optim_lite.RecReg.CanDt.SPN597', index=17,
      number=597, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN598', full_name='profi_optim_lite.RecReg.CanDt.SPN598', index=18,
      number=598, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN914', full_name='profi_optim_lite.RecReg.CanDt.SPN914', index=19,
      number=914, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN916', full_name='profi_optim_lite.RecReg.CanDt.SPN916', index=20,
      number=916, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN928', full_name='profi_optim_lite.RecReg.CanDt.SPN928', index=21,
      number=928, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN1624', full_name='profi_optim_lite.RecReg.CanDt.SPN1624', index=22,
      number=1624, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN1821', full_name='profi_optim_lite.RecReg.CanDt.SPN1821', index=23,
      number=1821, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='SPN1856', full_name='profi_optim_lite.RecReg.CanDt.SPN1856', index=24,
      number=1856, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=896,
  serialized_end=1333,
)

_RECREG_LLSDT = _descriptor.Descriptor(
  name='LLSDt',
  full_name='profi_optim_lite.RecReg.LLSDt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='TLLS1', full_name='profi_optim_lite.RecReg.LLSDt.TLLS1', index=0,
      number=1, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='CLLS1', full_name='profi_optim_lite.RecReg.LLSDt.CLLS1', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='FLLS1', full_name='profi_optim_lite.RecReg.LLSDt.FLLS1', index=2,
      number=3, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='TLLS2', full_name='profi_optim_lite.RecReg.LLSDt.TLLS2', index=3,
      number=4, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='CLLS2', full_name='profi_optim_lite.RecReg.LLSDt.CLLS2', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='FLLS2', full_name='profi_optim_lite.RecReg.LLSDt.FLLS2', index=5,
      number=6, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='TLLS3', full_name='profi_optim_lite.RecReg.LLSDt.TLLS3', index=6,
      number=7, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='CLLS3', full_name='profi_optim_lite.RecReg.LLSDt.CLLS3', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='FLLS3', full_name='profi_optim_lite.RecReg.LLSDt.FLLS3', index=8,
      number=9, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='TLLS4', full_name='profi_optim_lite.RecReg.LLSDt.TLLS4', index=9,
      number=10, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='CLLS4', full_name='profi_optim_lite.RecReg.LLSDt.CLLS4', index=10,
      number=11, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='FLLS4', full_name='profi_optim_lite.RecReg.LLSDt.FLLS4', index=11,
      number=12, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='TLLS5', full_name='profi_optim_lite.RecReg.LLSDt.TLLS5', index=12,
      number=13, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='CLLS5', full_name='profi_optim_lite.RecReg.LLSDt.CLLS5', index=13,
      number=14, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='FLLS5', full_name='profi_optim_lite.RecReg.LLSDt.FLLS5', index=14,
      number=15, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='TLLS6', full_name='profi_optim_lite.RecReg.LLSDt.TLLS6', index=15,
      number=16, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='CLLS6', full_name='profi_optim_lite.RecReg.LLSDt.CLLS6', index=16,
      number=17, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='FLLS6', full_name='profi_optim_lite.RecReg.LLSDt.FLLS6', index=17,
      number=18, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1336,
  serialized_end=1613,
)

_RECREG_LOG = _descriptor.Descriptor(
  name='LOG',
  full_name='profi_optim_lite.RecReg.LOG',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='Debug', full_name='profi_optim_lite.RecReg.LOG.Debug', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1615,
  serialized_end=1635,
)

_RECREG = _descriptor.Descriptor(
  name='RecReg',
  full_name='profi_optim_lite.RecReg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mID', full_name='profi_optim_lite.RecReg.mID', index=0,
      number=1, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='general', full_name='profi_optim_lite.RecReg.general', index=1,
      number=2, type=10, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='photo', full_name='profi_optim_lite.RecReg.photo', index=2,
      number=4, type=10, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nav', full_name='profi_optim_lite.RecReg.nav', index=3,
      number=5, type=10, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unidt', full_name='profi_optim_lite.RecReg.unidt', index=4,
      number=6, type=10, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='candt', full_name='profi_optim_lite.RecReg.candt', index=5,
      number=7, type=10, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='llsdt', full_name='profi_optim_lite.RecReg.llsdt', index=6,
      number=8, type=10, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='log', full_name='profi_optim_lite.RecReg.log', index=7,
      number=16, type=10, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_RECREG_GENERAL, _RECREG_PHOTO, _RECREG_NAV, _RECREG_UNIDT, _RECREG_CANDT, _RECREG_LLSDT, _RECREG_LOG, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=45,
  serialized_end=1635,
)

_RECREG_GENERAL.containing_type = _RECREG
_RECREG_PHOTO.containing_type = _RECREG
_RECREG_NAV.containing_type = _RECREG
_RECREG_UNIDT.containing_type = _RECREG
_RECREG_CANDT.containing_type = _RECREG
_RECREG_LLSDT.containing_type = _RECREG
_RECREG_LOG.containing_type = _RECREG
_RECREG.fields_by_name['general'].message_type = _RECREG_GENERAL
_RECREG.fields_by_name['photo'].message_type = _RECREG_PHOTO
_RECREG.fields_by_name['nav'].message_type = _RECREG_NAV
_RECREG.fields_by_name['unidt'].message_type = _RECREG_UNIDT
_RECREG.fields_by_name['candt'].message_type = _RECREG_CANDT
_RECREG.fields_by_name['llsdt'].message_type = _RECREG_LLSDT
_RECREG.fields_by_name['log'].message_type = _RECREG_LOG
DESCRIPTOR.message_types_by_name['RecReg'] = _RECREG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RecReg = _reflection.GeneratedProtocolMessageType('RecReg', (_message.Message,), {

  'General' : _reflection.GeneratedProtocolMessageType('General', (_message.Message,), {
    'DESCRIPTOR' : _RECREG_GENERAL,
    '__module__' : 'profi_optim_lite_pb2'
    # @@protoc_insertion_point(class_scope:profi_optim_lite.RecReg.General)
    })
  ,

  'Photo' : _reflection.GeneratedProtocolMessageType('Photo', (_message.Message,), {
    'DESCRIPTOR' : _RECREG_PHOTO,
    '__module__' : 'profi_optim_lite_pb2'
    # @@protoc_insertion_point(class_scope:profi_optim_lite.RecReg.Photo)
    })
  ,

  'NAV' : _reflection.GeneratedProtocolMessageType('NAV', (_message.Message,), {
    'DESCRIPTOR' : _RECREG_NAV,
    '__module__' : 'profi_optim_lite_pb2'
    # @@protoc_insertion_point(class_scope:profi_optim_lite.RecReg.NAV)
    })
  ,

  'UniDt' : _reflection.GeneratedProtocolMessageType('UniDt', (_message.Message,), {
    'DESCRIPTOR' : _RECREG_UNIDT,
    '__module__' : 'profi_optim_lite_pb2'
    # @@protoc_insertion_point(class_scope:profi_optim_lite.RecReg.UniDt)
    })
  ,

  'CanDt' : _reflection.GeneratedProtocolMessageType('CanDt', (_message.Message,), {
    'DESCRIPTOR' : _RECREG_CANDT,
    '__module__' : 'profi_optim_lite_pb2'
    # @@protoc_insertion_point(class_scope:profi_optim_lite.RecReg.CanDt)
    })
  ,

  'LLSDt' : _reflection.GeneratedProtocolMessageType('LLSDt', (_message.Message,), {
    'DESCRIPTOR' : _RECREG_LLSDT,
    '__module__' : 'profi_optim_lite_pb2'
    # @@protoc_insertion_point(class_scope:profi_optim_lite.RecReg.LLSDt)
    })
  ,

  'LOG' : _reflection.GeneratedProtocolMessageType('LOG', (_message.Message,), {
    'DESCRIPTOR' : _RECREG_LOG,
    '__module__' : 'profi_optim_lite_pb2'
    # @@protoc_insertion_point(class_scope:profi_optim_lite.RecReg.LOG)
    })
  ,
  'DESCRIPTOR' : _RECREG,
  '__module__' : 'profi_optim_lite_pb2'
  # @@protoc_insertion_point(class_scope:profi_optim_lite.RecReg)
  })
_sym_db.RegisterMessage(RecReg)
_sym_db.RegisterMessage(RecReg.General)
_sym_db.RegisterMessage(RecReg.Photo)
_sym_db.RegisterMessage(RecReg.NAV)
_sym_db.RegisterMessage(RecReg.UniDt)
_sym_db.RegisterMessage(RecReg.CanDt)
_sym_db.RegisterMessage(RecReg.LLSDt)
_sym_db.RegisterMessage(RecReg.LOG)


_RECREG_CANDT.fields_by_name['SPN582']._options = None
_RECREG_CANDT.fields_by_name['SPN928']._options = None
_RECREG.fields_by_name['mID']._options = None
# @@protoc_insertion_point(module_scope)



from omnicomm.types import RegFwCmd

COMMAND_PROTO: dict[int | RegFwCmd, str] = {
    0x86: 'omnicomm.proto.profi_optim_lite_pb2.RecReg',
    0x95: 'omnicomm.proto.profi2_optim2_lite2_pb2.RecReg',
    # (202000013, 114, 0x86): 'omnicomm.proto.profi2_optim2_lite2_pb2.RecReg',
    # (202000013, 0x86): 'omnicomm.proto.profi2_optim2_lite2_pb2.RecReg',
}



import struct

import libscrc

from omnicomm import settings
from omnicomm.commands import BaseCommand, commands
from omnicomm.exceptions import (CRCDoesNotMatchError, EmptyDataError,
                                 FrameMarkerDoesNotExistError,
                                 UnpackingCRCError)
from omnicomm.registry import reg_fw_cmd
from omnicomm.types import RegFwCmd
from omnicomm.utils import import_string

FRAME_MARKER = b'\xc0'


class Protocol:
    @classmethod
    def pack_crc(cls, crc: int) -> bytes:
        raise NotImplementedError

    @classmethod
    def unpack_crc(cls, data: bytes, offset: int = 0) -> int:
        raise NotImplementedError

    @classmethod
    def decode(cls, data) -> bytes:
        return data[1:].replace(b'\xdb\xdc', FRAME_MARKER).replace(b'\xdb\xdd', b'\xdb')

    @classmethod
    def encode(cls, data) -> bytes:
        return FRAME_MARKER + data.replace(b'\xdb', b'\xdb\xdd').replace(FRAME_MARKER, b'\xdb\xdc')

    @classmethod
    def make_crc(cls, cmd_id: int, length, data: bytes) -> int:
        crc16 = libscrc.xmodem(struct.pack('<BH', cmd_id, length) + data, 0xffff)
        return crc16

    @classmethod
    def pack(cls, cmd: BaseCommand) -> bytes:
        data = cmd.pack()
        length = len(data)
        crc = cls.make_crc(cmd.id, length, data)
        data = struct.pack('<B', cmd.id) + struct.pack('<H', length) + data
        data += cls.pack_crc(crc)
        return cls.encode(data)

    @classmethod
    def unpack(cls, data: bytes) -> tuple[BaseCommand, bytes]:
        if not data:
            msg = 'Data does not exist'
            raise EmptyDataError(msg)
        elif data[0] != FRAME_MARKER[0]:
            msg = 'Frame marker does not exist'
            raise FrameMarkerDoesNotExistError(msg)

        another_frame_idx = data[1:].find(FRAME_MARKER)
        remain = data[another_frame_idx + 1:] if another_frame_idx > -1 else b''

        data = cls.decode(data[:another_frame_idx + 1] if another_frame_idx > -1 else data)

        cmd_num = struct.unpack_from('<B', data, offset=0)[0]
        length = struct.unpack_from('<H', data, offset=1)[0]
        try:
            original_crc = cls.unpack_crc(data, offset=3 + length)
        except struct.error as err:
            msg = 'More data is required to unpack'
            raise UnpackingCRCError(msg) from err

        original_data = data[3: 3 + length]

        crc = cls.make_crc(cmd_num, length, original_data)
        if original_crc != crc:
            msg = 'CRC does not match'
            raise CRCDoesNotMatchError(msg)

        cmd = commands[cmd_num].unpack(original_data)
        return cmd, remain

    @classmethod
    def unpack_many(cls, data: bytes) -> tuple[[BaseCommand], bytes]:
        cmds = []
        remain = b''
        try:
            while 1:
                cmd, remain = cls.unpack(data)
                cmds.append(cmd)
                data = remain
        except Exception as err:
            if not cmds:
                raise err
        return cmds, remain

    @classmethod
    def register_proto(cls, item: RegFwCmd, module: str) -> None:
        proto_class = import_string(module)
        reg_fw_cmd.register(item, proto_class)

    @classmethod
    def load_command_proto(cls, cmd_proto: dict[int | RegFwCmd, str] | None = None) -> None:
        _cmd_proto: dict[int | RegFwCmd, str] = getattr(settings, 'COMMAND_PROTO', {}) or {}
        _cmd_proto.update(cmd_proto or {})
        reg_fw_cmd.clear()
        for k, v in _cmd_proto.items():
            cls.register_proto(k, v)


class RegistrarProtocol(Protocol):
    @classmethod
    def pack_crc(cls, crc: int) -> bytes:
        return struct.pack('>H', crc)

    @classmethod
    def unpack_crc(cls, data: bytes, offset: int=0) -> int:
        return struct.unpack_from('<H', data, offset=offset)[0]


class ServerProtocol(Protocol):
    @classmethod
    def pack_crc(cls, crc: int) -> bytes:
        return struct.pack('<H', crc)

    @classmethod
    def unpack_crc(cls, data: bytes, offset: int = 0) -> int:
        return struct.unpack_from('>H', data, offset=offset)[0]



import struct
import time

from google.protobuf.json_format import MessageToDict, ParseDict
from google.protobuf.message import Message

from omnicomm.exceptions import CommandAlreadyExistsError
from omnicomm.registry import reg_fw_cmd

Command = type['BaseCommand']


class BaseCommand:
    id: int = 0  # noqa: A003
    format: str = ''  # noqa: A003

    BASE_TIME: int = 1230768000

    def __init__(self, value: dict) -> None:
        self.value: dict = value

    @classmethod
    def to_unix_time(cls, omnicomm_time: int) -> int:
        return cls.BASE_TIME + omnicomm_time

    @classmethod
    def to_omnicomm_time(cls, unix_time: int | None) -> int:
        if unix_time is None:
            unix_time = int(time.time())
        omnicomm_time = unix_time - cls.BASE_TIME
        if omnicomm_time < 0:
            omnicomm_time = 0
        return omnicomm_time

    @classmethod
    def pack_protobuf(cls, value: dict, proto_class: type[Message]) -> bytes:
        proto_instance: Message = ParseDict(value, proto_class())
        return proto_instance.SerializeToString()

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:
        raise NotImplementedError()

    def pack(self, conf: dict | None = None) -> bytes:
        return struct.pack(f'<{self.format}', *self.from_dict(self.value, conf or {}))

    @classmethod
    def unpack_protobuf(cls, data: bytes, proto_class: type[Message]):
        proto_instance = proto_class()
        proto_instance.ParseFromString(data)
        return MessageToDict(proto_instance)

    @classmethod
    def to_dict(cls, value: tuple[int, ...], conf: dict) -> dict:
        raise NotImplementedError()

    @classmethod
    def unpack(cls, data: bytes, conf: dict | None = None) -> 'BaseCommand':
        return cls(cls.to_dict(struct.unpack_from(cls.format, data), conf or {}))


class Cmd80(BaseCommand):
    id: int = 0x80  # noqa: A003
    format: str = 'II'  # noqa: A003

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:  # noqa: ARG003
        return value.get('reg_id', 0) or 0, value.get('firmware', 0) or 0

    @classmethod
    def to_dict(cls, value: tuple[int, ...], conf: dict) -> dict:  # noqa: ARG003
        return dict(reg_id=value[0], firmware=value[1])


class Cmd81(BaseCommand):
    id: int = 0x81  # noqa: A003
    format: str = 'II'  # noqa: A003

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:  # noqa: ARG003
        return value.get('server_id', 0) or 0, value.get('server_ver', 0) or 0


class Cmd85(BaseCommand):
    id: int = 0x85  # noqa: A003
    format: str = 'I'  # noqa: A003

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:  # noqa: ARG003
        return (value.get('rec_id', 0) or 0,)

    @classmethod
    def to_dict(cls, value: tuple[int, ...], conf: dict) -> dict:  # noqa: ARG003
        return dict(rec_id=value[0])


class Cmd86(BaseCommand):
    id: int = 0x86  # noqa: A003
    format: str = 'IIB'  # noqa: A003

    # @classmethod
    # def to_datetime(cls, omnicomm_time):
    #     return datetime(*tuple(time.gmtime(cls.to_unix_time(omnicomm_time)))[:6])

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:  # noqa: ARG003
        omnicomm_time = value.get('omnicomm_time', 0) or cls.to_omnicomm_time(value.get('unix_time', None))
        return value['rec_id'], int(omnicomm_time), value['priority']

    def pack(self, conf: dict | None = None) -> bytes:
        msgs: list | None = self.value.get('msgs')
        if not msgs:
            return b''

        reg_id, fw = self.value.get('reg_id', 0), self.value.get('firmware', 0)
        proto_class: type[Message] = reg_fw_cmd[(reg_id, fw, self.id)]

        data = b''
        for msg in msgs:
            _msg = self.pack_protobuf(msg, proto_class)
            length = len(_msg)
            data += struct.pack('<H', length)
            data += _msg

        return struct.pack(f'<{self.format}', *self.from_dict(self.value, conf or {})) + data

    @classmethod
    def to_dict(cls, value: tuple[int, ...], conf: dict) -> dict:  # noqa: ARG003
        return dict(rec_id=value[0], omnicomm_time=value[1], priority=value[2], unix_time=cls.to_unix_time(value[1]))

    @classmethod
    def unpack(cls, data: bytes, conf: dict | None = None) -> BaseCommand:
        if not data:
            return cls({})

        rec_id, omnicomm_time, priority = struct.unpack_from(cls.format, data)
        value: dict = cls.to_dict(
            (
                rec_id,
                omnicomm_time,
                priority,
            ),
            conf or {},
        )

        data = data[9:]

        reg_id, fw = (conf or {}).get('reg_id', 0), (conf or {}).get('firmware', 0)
        proto_class: type[Message] = reg_fw_cmd[(reg_id, fw, cls.id)]

        msgs = []
        while data:
            length = struct.unpack_from('<H', data)[0]
            proto_data = cls.unpack_protobuf(data[2 : 2 + length], proto_class)
            msgs.append(proto_data)
            data = data[2 + length :]

        value.update(dict(msgs=msgs))
        return cls(value)


class Cmd87(BaseCommand):
    id: int = 0x87  # noqa: A003
    format: str = 'I'  # noqa: A003

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:  # noqa: ARG003
        return (value.get('rec_id', 0) or 0,)

    @classmethod
    def to_dict(cls, value: tuple[int, ...], conf: dict) -> dict:  # noqa: ARG003
        return dict(rec_id=value[0])


class Cmd88(BaseCommand):
    id: int = 0x88  # noqa: A003
    format: str = 'I'  # noqa: A003

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:  # noqa: ARG003
        return (value.get('rec_id', 0) or 0,)

    @classmethod
    def to_dict(cls, value: tuple[int, ...], conf: dict) -> dict:  # noqa: ARG003
        return dict(rec_id=value[0])


# class Cmd89(BaseCommand):
#     id: int = 0x89
#
#
# class Cmd8A(BaseCommand):
#     id: int = 0x8A
#
#
# class Cmd8B(BaseCommand):
#     id: int = 0x8B
#
#
# class Cmd8C(BaseCommand):
#     id: int = 0x8C
#
#
# class Cmd8D(BaseCommand):
#     id: int = 0x8D
#
#
# class Cmd8E(BaseCommand):
#     id: int = 0x8E
#
#
# class Cmd8F(BaseCommand):
#     id: int = 0x8F
#
#
# class Cmd90(BaseCommand):
#     id: int = 0x90
#
#
# class Cmd91(BaseCommand):
#     id: int = 0x91
#
#
# class Cmd92(BaseCommand):
#     id: int = 0x92


class Cmd93(BaseCommand):
    id: int = 0x93  # noqa: A003

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:  # noqa: ARG003
        return tuple()

    @classmethod
    def to_dict(cls, value: tuple[int, ...], conf: dict) -> dict:  # noqa: ARG003
        return {}


class Cmd94(BaseCommand):
    id: int = 0x94  # noqa: A003
    format: str = 'I'  # noqa: A003

    @classmethod
    def from_dict(cls, value: dict, conf: dict) -> tuple[int, ...]:  # noqa: ARG003
        omnicomm_time = value.get('omnicomm_time', 0) or cls.to_omnicomm_time(value.get('unix_time', None))
        return (int(omnicomm_time),)

    @classmethod
    def to_dict(cls, value: tuple[int, ...], conf: dict) -> dict:  # noqa: ARG003
        return dict(omnicomm_time=value[0], unix_time=cls.to_unix_time(value[0]))


class Cmd95(Cmd86):
    id: int = 0x95  # noqa: A003


# class Cmd96(BaseCommand):
#     id: int = 0x96
#
#
# class Cmd97(BaseCommand):
#     id: int = 0x97
#
#
# class Cmd98(BaseCommand):
#     id: int = 0x99
#
#
# class Cmd99(BaseCommand):
#     id: int = 0x99
#
#
# class Cmd9A(BaseCommand):
#     id: int = 0x9A
#
#
# class Cmd9B(BaseCommand):
#     id: int = 0x9B
#
#
# class Cmd9C(BaseCommand):
#     id: int = 0x9C
#
#
# class Cmd9D(BaseCommand):
#     id: int = 0x9D
#
#
# class Cmd9E(BaseCommand):
#     id: int = 0x9E


class Cmd9F(Cmd86):
    id: int = 0x9F  # noqa: A003


class CmdA0(Cmd87):
    id: int = 0xA0  # noqa: A003


commands = {}
_command_classes = BaseCommand.__subclasses__()
while _command_classes:
    _cmd = _command_classes.pop(0)
    if _cmd.id in commands:
        msg = 'Command ID already exists'
        raise CommandAlreadyExistsError(msg)
    commands[_cmd.id] = _cmd
    _command_classes.extend(_cmd.__subclasses__())


__all__ = tuple([cmd.__name__ for cmd in commands.values()])



from unittest import TestCase
from unittest.mock import Mock

from omnicomm.exceptions import ProtoDoesNotExistError
from omnicomm.registry import reg_fw_cmd


class ProtocolTest(TestCase):
    def test_register_unregister_command(self):
        reg_fw_cmd.register(42, Mock)
        self.assertTrue(reg_fw_cmd[42] is Mock)
        reg_fw_cmd.unregister(42)
        with self.assertRaises(ProtoDoesNotExistError):
            reg_fw_cmd[42]

    def test_clear_the_register(self):
        reg_fw_cmd.register(42, Mock)
        self.assertTrue(reg_fw_cmd._registry)
        reg_fw_cmd.clear()
        self.assertTrue(not reg_fw_cmd._registry)

    def test_a_command_registering(self):
        reg_fw_cmd.register(42, Mock)
        self.assertTrue(reg_fw_cmd[42] is Mock)
        reg_fw_cmd.register((0, 42), Mock)
        self.assertTrue(reg_fw_cmd[42] is Mock)
        reg_fw_cmd.register((0, 0, 42), Mock)
        self.assertTrue(reg_fw_cmd[42] is Mock)

        reg_fw_cmd.register((1, 42), Mock)
        self.assertTrue(reg_fw_cmd[(1, 42)] is Mock)
        reg_fw_cmd.register((0, 1, 42), Mock)
        self.assertTrue(reg_fw_cmd[(1, 42)] is Mock)

        reg_fw_cmd.register((1, 1, 42), Mock)
        self.assertTrue(reg_fw_cmd[(1, 1, 42)] is Mock)



from google.protobuf.message import Message

from omnicomm.exceptions import ProtoDoesNotExistError
from omnicomm.types import RegFwCmd


class RegFwCmdRegistry:
    def __init__(self) -> None:
        self._registry: dict[int | RegFwCmd, type[Message]] = {}

    def __getitem__(self, item: int | RegFwCmd) -> type[Message]:
        proto_cls: type[Message] | None = None
        if isinstance(item, int):
            proto_cls = self._registry.get(item, None)
        elif isinstance(item, tuple):
            key: RegFwCmd = tuple(filter(bool, item))
            if len(key) == 3:  # noqa: PLR2004
                reg_id, fw, cmd = key
                proto_cls = (
                    self._registry.get(key, None)
                    or self._registry.get((reg_id, cmd), None)
                    or self._registry.get(cmd, None)
                )
            elif len(key) == 2:  # noqa: PLR2004
                reg_id, cmd = key
                proto_cls = self._registry.get(key, None) or self._registry.get(cmd, None)
            elif len(key) == 1:
                proto_cls = self._registry.get(key[0], None)

        if proto_cls is None:
            msg = 'Proto file does not exist'
            raise ProtoDoesNotExistError(msg)

        return proto_cls

    def clear(self):
        self._registry = {}

    def register(self, key: int | RegFwCmd, proto_class: type[Message]) -> None:
        self._registry[key] = proto_class

    def unregister(self, key: int | RegFwCmd) -> None:
        self._registry.pop(key, None)


reg_fw_cmd = RegFwCmdRegistry()



import sys
from importlib import import_module

from omnicomm.types import RegFwCmd


def cached_import(module_path, class_name):
    # Check whether module is loaded and fully initialized.
    if not (
        (module := sys.modules.get(module_path))
        and (spec := getattr(module, "__spec__", None))
        and getattr(spec, "_initializing", False) is False
    ):
        module = import_module(module_path)
    return getattr(module, class_name)


def import_string(dotted_path):
    """
    Import a dotted module path and return the attribute/class designated by the
    last name in the path. Raise ImportError if the import failed.
    """
    try:
        module_path, class_name = dotted_path.rsplit(".", 1)
    except ValueError as err:
        raise ImportError("%s doesn't look like a module path" % dotted_path) from err

    try:
        return cached_import(module_path, class_name)
    except AttributeError as err:
        msg = f'Module "{module_path}" does not define a "{class_name}" attribute/class'
        raise ImportError(msg) from err


def cast_command_proto(command_proto: dict[str, str]) -> dict[int | RegFwCmd, str]:
    """Cast a command`s map from a config with `str` keys to config with `int | tuple[int, ...]` keys.

    For example:

    ```yaml
    ---
    COMMAND_PROTO:
      (200000000, 134): omnicomm.proto.profi2_optim2_lite2_pb2.RecReg
      '134': omnicomm.proto.profi_optim_lite_pb2.RecReg
    ```

    ```python
    import settings

    command_proto = cast_command_proto(settings.COMMAND_PROTO)
    assert command_proto[134]
    assert command_proto[(200000000, 134)]
    ```
    """
    conf: dict[int | RegFwCmd, str] = {}
    for k, v in command_proto.items():
        key = tuple(map(int, k.replace(' ', '').strip('()').split(',')))
        if len(key) == 1:
            conf[key[0]] = v.strip()
        else:
            conf[key] = v.strip()
    return conf



from unittest import TestCase

from omnicomm.utils import cast_command_proto


class UtilsTest(TestCase):
    def test_cast_command_proto(self):
        command_proto = cast_command_proto(
            {
                '123': 'path.to.proto1',
                '(1, 23)': 'path.to.proto2',
                '(1, 2, 3)': 'path.to.proto3',
            }
        )
        self.assertTrue(command_proto[123] == 'path.to.proto1')
        self.assertTrue(command_proto[(1, 23)] == 'path.to.proto2')
        self.assertTrue(command_proto[(1, 2, 3)] == 'path.to.proto3')
